#############################################################################
##
## Code to document Section 5 of 
## "Resolving a Multi-Million Dollar Contract Dispute with a Latin Square"
## v35
##
#############################################################################

#####################################
## Note that in lines 189-191 
## to get three different levels of informativeness of the prior, change the value 
## of the constant "cons.c" to get (1) the primary informative
## prior (const.c = 1), (2) the less informative prior with double the standard 
## deviation of the primary one (const.c = 1/2), or (3) the prior with very large standard 
## deviations, corresponding to a very non-informative prior (const.c = 1/100,000,000).
#####################################


# Read in data and set random seed
set.seed(10)
library(MASS)

trucks_data = read.csv("truck_data2.csv")

attach(trucks_data)

# Set n equal to the number of truck runs
n = dim(trucks_data)[1]

# Set y equal to the tons column of the data
y = trucks_data$tons


#####################################
# Set up design matrix for route, day, and truck effects. 
# Use sum contrasts.  1 if in that level, otherwise 0, except
# for last entry, which has -1 for all dummy variables.  
#####################################

X = matrix(0,n,18)
colnames(X) <- c("Intercept", "Route1", "Route2", "Route3", "Route4", "Mon", "Tues", "Wed", "Thurs", "A1", "A2",
                 "B1", "B2", "C1", "C2", "D1", "D2", "E1")

##  Intercept, an estimate of the grand mean with these contrasts
X[,1] = 1    
##  Route
for(i in 1:50)
  ifelse(route[i]==1, X[i,2] <- 1, 
         ifelse(route[i]==2, X[i,3] <- 1, 
                ifelse(route[i]==3, X[i,4] <- 1, 
                       ifelse(route[i]==4, X[i,5] <- 1,
                              X[i,2] <- X[i,3] <- X[i,4] <- X[i,5] <- -1
                       ))))
##  Check
cbind(X[,2:5], route)

##  Day
for(i in 1:50)
  ifelse(day[i]=="M", X[i,6] <- 1, 
         ifelse(day[i]=="T", X[i,7] <- 1, 
                ifelse(day[i]=="W", X[i,8] <- 1, 
                       ifelse(day[i]=="TH", X[i,9] <- 1,
                              X[i,6] <- X[i,7] <- X[i,8] <- X[i,9] <- -1
                       ))))
##  Check
cbind(X[,6:9], day)

##  Truck

for(i in 1:50)
  ifelse(truck[i]=="A1", X[i,10] <- 1, 
         ifelse(truck[i]=="A2", X[i,11] <- 1, 
                ifelse(truck[i]=="B1", X[i,12] <- 1, 
                       ifelse(truck[i]=="B2", X[i,13] <- 1,
                              ifelse(truck[i]=="C1", X[i,14] <- 1,  
                                     ifelse(truck[i]=="C2", X[i,15] <- 1, 
                                            ifelse(truck[i]=="D1", X[i,16] <- 1, 
                                                   ifelse(truck[i]=="D2", X[i,17] <- 1,
                                                          ifelse(truck[i]=="E1", X[i,18] <- 1, 
                                                                 X[i,10:18]  <- -1
                                                          )))))))))
##  Check
cbind(X[,10:18], truck)



#####################################
# Functions for draws of beta from posterior multivariate normal 
# and sigmasq from gamma. (beta (B) and sigmasq (V) in paper)
#####################################

estim_beta = function(y,mu0,Lambda0,sigsq){
  Lambda_n = t(X)%*%X+Lambda0
  mu_n = solve(Lambda_n)%*%(t(X)%*%y + Lambda0%*%mu0)
  newbeta = mvrnorm(1,mu_n,sigsq*solve(Lambda_n))
  return(newbeta)
}

estim_sigmasq = function(y,a0,b0,mu0,Lambda0){
  an = .5*n+a0
  Lambda_n = t(X)%*%X+Lambda0
  mu_n = solve(Lambda_n)%*%(t(X)%*%y + Lambda0%*%mu0)  
  bn = b0 + .5*(t(y)%*%y + t(mu0)%*%Lambda0%*%mu0 - t(mu_n)%*%Lambda_n%*%mu_n)
  newsigsq = 1/rgamma(1,an,bn)
  return(newsigsq)
}



#############################################################################
## Section 5.2
#############################################################################

#####################################
## Table 8. Prior mean and standard deviation of the parameters of the mean model at
## expression (1).
#####################################

#####################################
#   Specify prior mean and variance
#####################################

##  The prior mean (the mu0 vector in expression (7) in the paper) for the beta vector  
##  has first element the grand mean (12 tons for the city prior and 13 tons for 
##  the manufacturer), followed by zeroes for the next 17 elements corresponding 
##  to the 17 independent effects.

mu0 = rep(0,18)                 ##  Initialize mu0 to all zeroes in the prior
## 18 = 1 (mu) + 4 (route) + 4 (day) + 9 (truck) (# of independent parameters)

##  First element of mu0 gets changed, one at a time, to the 
##  elements of "prior_vec" below. Ignore the unused case for prior value of 12.5.

prior_vec = c(12, 12.5, 13)

## Write the inverse of the covariance matrix as Lambda0 = cons.c*M for a 
## constant, cons.c--an "informativeness parameter"--and a matrix M.
## Note that the covariance matrix is then (Lambda0)^-1 = (cons.c*M)^-1 = cons.c^-1 * M^-1.
## This shows cons.c^-1 can be chosen as the standard error of estimate if M is appropriately defined [?]
## To get posterior densities of beta
## under 3 different choices for the informativeness parameter const.c in decreasing order of informativeness:
## 
## Case (1): cons.c = 1
## Case (2): cons.c = 1/2
## Case (3): cons.c = 1/100000000


## Values of prior variances to get the last column of Table 8

##  Choose the variances along the diagonal of the matrix M to be the squares 
##  of the following standard deviations:
##  grand mean (mu) 0.5 tons
##	route (gamma), 1 ton
##	day (tau), 1 ton
##	truck (rho), 0.1 ton
##  Set covariances--the off-diagonal elements of M--equal to 0.

## Define cases also depending on the variance of the truck effects--but usually given cons.c = 1.
## That variance appears in the diagonal elements of rows 10 through 18 in M. Here is the parameter:
sd(aggregate(trucks_data$tons, by=list(trucks_data$truck), FUN=mean)[,2])

truck.sd = sd(aggregate(trucks_data$tons, by=list(trucks_data$truck), FUN=mean)[,2])

M <- rep(0, 18*18)
dim(M) <- c(18,18)
M[1,1] = (1/0.5)^2
M[2,2] = 1
M[3,3] = 1
M[4,4] = 1
M[5,5] = 1
M[6,6] = 1
M[7,7] = 1
M[8,8] = 1
M[9,9] = 1
M[10,10] = truck.sd^2
M[11,11] = truck.sd^2
M[12,12] = truck.sd^2
M[13,13] = truck.sd^2
M[14,14] = truck.sd^2
M[15,15] = truck.sd^2
M[16,16] = truck.sd^2
M[17,17] = truck.sd^2
M[18,18] = truck.sd^2


## Trucks are in order A1, A2, B1, B2, etc.

## Make one of three choices for the informativeness parameter, cons.c, to determine level of 
## informativeness sought for the prior. See lines 10-15.
cons.c = 1
## cons.c = 1/2
## cons.c = 1/100000000

Lambda0 = cons.c*M 

##  Since Lambda0 is the inverse of the covariance matrix, setting cons.c to be large
##  means prior variances are small, corresponding to greater certainty in the 
##  prior.  The prior dominates. Setting cons.c makes the 
##  prior variance large.  A large variance means less certainty, and 
##  this allows the data to dominate. e.g., setting cons.c = 1/100,000,000 
##  gives posterior results that are very close to the results from the ordinary ANOVA.  
##  The data dominates.

##  Parameters of the prior on the standard error of estimate, sigmasq, 
##  in expression (8) of the paper are: 
a0 = 1
b0 = 1




#############################################################################
## Section 5.3
#############################################################################

#####################################
#  Gibbs sampler for drawing 5000 samples of 
#  B's (mean vector) and V's (standard error of estimate)
#####################################

numiter = 5000

results_array = array(0,c(numiter,3,18))

for(j in 1:3){
  mu0[1] = prior_vec[j]
  betamat = matrix(0,numiter,18)
  sigsqmat = rep(0,numiter)
  
  betamat[1,] = mu0
  sigsqmat[1] = .1
  
  betaold = betamat[1,]
  sigsqold = sigsqmat[1]
  for(i in 2:numiter){
    betaold = estim_beta(y,mu0,Lambda0,sigsqold)
    sigsqold = estim_sigmasq(y,a0,b0,mu0,Lambda0)
    
    betamat[i,] = betaold
    sigsqmat[i] = sigsqold
  }
  results_array[,j,] = betamat
}

##  Throw away the first 1000
results_array = results_array[1001:numiter , , ]



## 2nd para. of Section 5.3
## Posterior mean of grand mean for each of the two priors on the grand mean (city and manufacturer)

cons.c  ## Informativeness parameter selected above
posterior.means <- round(apply(results_array, c(2,3), mean),2)    ## Used in Figure 6.
posterior.means  ## 3x18 matrix used in Figure 6. {Trucks are in order A1, A2, B1, B2, etc.}


## Compare posterior truck effects to truck effects under 3-way fixed effects ANOVA model.

## Posterior truck effects under Bayesian model
cons.c
p.m <- posterior.means[1,10:18]
p.m

## Truck effects under fixed effects ANOVA model (in order A1, A2, etc. through E2)
aov.three.way <- aov(tons ~ route + day + truck, data=trucks_data)
f.m <- model.tables(aov.three.way)[[1]][[3]]
f.m

difference = f.m[1:9] - p.m
round(difference[1:9], 3)  ## difference is close to 0 for all 9 truck parameters

##{N.B. Note that the posterior truck effects are practically identical to the fixed model
## truck effects--regardless of the choice of the informativeness parameter, cons.c. That is,
## there is no shrinkage. 

## Question. Why? I expected shrinkage of the fixed effects towards the prior mean truck
## effects, which are 0. The weights would be 50 for the fixed effects and 1/(0.01^2) = 100
## for the prior mean truck effects (since the prior for the truck effects has s.d. of 0.1)} 


cons.c
posterior.means[1,1] == 13.58  ## posterior mean of grand mean 
## with city prior on grand mean of 12 tons and cons.c = 1

posterior.means[3,1] == 13.66  ## posterior mean of grand mean 
## with manufacturer prior on grand mean of 13 tons and cons.c = 1

## These last two values = 13.71 if cons.c = 1/100000000



# p. 21 last full para. 
#####################################
# Get the posterior grand means under the city's and the manufacturer's prior, respectively,
## as weighted averages of the data mean and the prior means.
#####################################

# The data mean is 13.71 tons. The data information is 50 and the prior
# information is 4, giving weights of 50/54 and 4/54, or 
# 0.9259259 and 0.07407407, respectively. The posterior means of the grand mean, mu,
## under the city's and the manufacturer's priors are then:

round((50/54)*13.71 + (4/54)*12, 2) == 13.58  # city posterior mean = 13.58333

round((50/54)*13.71 + (4/54)*13, 2) == 13.66  # manufacturer posterior mean = 13.65741


#####################################
## {Less important question. 
## The Associate Editor's comment 6 says the "data" weight is 50 and 
## the "prior weight is 4. However, there is a minor discrepancy between these values
## and the values obtained from a direct calculation of the weights--as follows.
## The posterior mean equals a weighted average of the prior mean and the data mean.
## The weight for the prior mean (for cons.c = 1) is the reciprocal of its variance, 
## i.e., 1/0.25 = 4, which agrees with the Associate Editor's value of 4. However, 
## the weight for the data mean is the reciprocal of its variance, 
## i.e., 1/0.01916122 = 52.18873, where 0.01916122 = var(y)/(50). But 52.18873 
## does not agree exactly with the Associate Ediotor's value of 50.}
#####################################



#####################################
# Figure 6 Prior and posterior densities for the mean load
#####################################

library(ggplot2)
library(grid)

plot_prior.normal.distributions <- (
  
  ggplot(data.frame(x = c(5, 20)), aes(x)) 
  + stat_function(fun = dnorm, args = list(mean = prior_vec[1], sd = 0.5), linetype="dashed", n=10000)  
  + stat_function(fun = dnorm, args = list(mean = prior_vec[3], sd = 0.5), linetype="dashed", n=10000)
  + stat_function(fun = dnorm, args = list(mean = posterior.means[1,1], sd = posterior.sds[1,1]), linetype="solid", n=10000)  
  + stat_function(fun = dnorm, args = list(mean = posterior.means[3,1], sd = posterior.sds[3,1]), linetype="solid", n=10000)  
  
  +scale_x_discrete(name="Grand mean load")
  +scale_y_continuous(name=" ")
  +coord_cartesian(xlim=c(10.75,14.25), ylim=c(0,5.25))
  +theme(axis.text=element_text(size=17,color="black"),
         axis.text.x=element_text(size=24, hjust=0.5),
         axis.text.y=element_text(size=24, hjust=0.5),
         axis.title=element_text(size=30,color=("black")),
         panel.background=element_blank(),
         panel.grid.major.y=element_line(color=rgb(.7,.7,.7),size=.01),
         panel.grid.minor.y=element_line(color=rgb(.9,.9,.9),size=.001),
         panel.grid.major.x=element_blank(),
         axis.line=element_line(size=.5),
         title=element_text(size=20,family="serif")
  )
  
  + geom_segment(aes(x = 11.35, y = 1, xend = 11.5, yend = 0.7), arrow = arrow(length = unit(0.5, "cm")))
  + geom_segment(aes(x = 12.5, y = 1.5, xend = 12.75, yend = 0.8), arrow = arrow(length = unit(0.5, "cm")))
  + annotate("text", x = 11.35, y = 1.25, label = "City prior", size=6)
  + annotate("text", x = 12.5, y = 1.75, label = "Manufacturer prior", size=6)
  + geom_segment(aes(x = 12.5, y = 3.55, xend = 13.48, yend = 3.25), arrow = arrow(length = unit(0.5, "cm")))
  + geom_segment(aes(x = 12.5, y = 4.55, xend = 13.58, yend = 4.25), arrow = arrow(length = unit(0.5, "cm")))
  + annotate("text", x = 12.5, y = 3.8, label = "City posterior", size=6)
  + annotate("text", x = 12.5, y = 4.8, label = "Manufacturer posterior", size=6)
)

plot_prior.normal.distributions


## {N.B. Table 10 is to be re-numbered to be Table 9}

#####################################
## Table 10. Posterior probabilities for the rst set of ve trucks that loads will be 12.5 or
## more tons
#####################################

#####################################
# Posterior probabilities that loads will be 12.5 or more tons by route and day under each prior 
#####################################

##  For each iteration, get the predicted posterior tons for each truck, for each prior.

# Storing posterior predicted tons for the city prior of 12
tons.12 <- matrix(0, nrow=50, ncol=(numiter-1000))
for(i in 1:(numiter-1000))
  tons.12[,i] <- X%*%results_array[i,1,]

# Storing posterior predicted tons for the manufacturer prior of 13
tons.13 <- matrix(0, nrow=50, ncol=(numiter-1000))
for(i in 1:(numiter-1000))
  tons.13[,i] <- X%*%results_array[i,3,]

##  Now, can count over any index (route, day, truck),
##  or any combination of these, to see what proportion 
##  have predicted tons over 12.5

##  Count over each route/day/truck combination (50 values, for each prior)

# Probability of greater than or equal to 12.5 for each of the 50 truck runs (12 prior)
over.12.5.given.12 <- numeric(50)
for(i in 1:50)
  over.12.5.given.12[i] <- length(tons.12[i,tons.12[i,]>=12.5])/(numiter-1000)


# Probability of greater than or equal to 12.5 for each of the 50 truck runs (13 prior)
over.12.5.given.13 <- numeric(50)
for(i in 1:50)
  over.12.5.given.13[i] <- length(tons.13[i,tons.13[i,]>=12.5])/(numiter-1000)


prob.over.12.5 <- cbind(trucks_data[,2:4], over.12.5.given.12, over.12.5.given.13)

##  The result, "prob.over.12.5", is a data frame.
##  The first three columns identify the day/route/truck in that order.
##  The fourth column gives the probability of a load over 12.5, 
##  given a prior of 12 for the grand mean.
##  The fifth column gives the probability of a load over 12.5, 
##  given a prior of 13 for the grand mean.

##  Obtain the probability of tons being greater than or equal to 12.5 
##  by day and route (averaged over trucks).
##  These provide the values for Table 10.

##  For the prior in which the prior on the grand mean is 12 tons.
##  Obtain the mean of the 4th column (over.12.5.given.12) by unique combinations 
##  of the first two columns (route, day)
temp <-t(tapply(prob.over.12.5[,4], prob.over.12.5[,1:2], mean))  
   ## 5x5 matrix of probs by day and route

##  Since the order of the output by Day is F, M, Tu, Th, W 
##  change the order to be M-F. 


## Table 10 is (with city prior):

tempm <- as.vector(temp[, 2])
temptu <- as.vector(temp[, 3])
tempw <- as.vector(temp[, 5])
tempth <- as.vector(temp[, 4])
tempf <- as.vector(temp[, 1])
round(cbind(tempm, temptu, tempw, tempth, tempf), 2)


## For the manufacturer prior in which the prior on the grand mean is 13 tons.
## No table of values is given in the paper for these results.
# Obtain the mean of the 5th column by unique combinations of the first two columns:
temp <- t(tapply(prob.over.12.5[,5], prob.over.12.5[,1:2], mean))
tempm <- as.vector(temp[, 2])
temptu <- as.vector(temp[, 3])
tempw <- as.vector(temp[, 5])
tempth <- as.vector(temp[, 4])
tempf <- as.vector(temp[, 1])
round(cbind(tempm, temptu, tempw, tempth, tempf), 2)


