#############################################################################
##
## Purpose: Document Figures, Tables, and text of Sections 1-4,
## "Resolving a Multi-Million Dollar Contract Dispute with a Latin Square"
##
#############################################################################


###################################
# Read In Data
###################################
library(ggplot2)
library(grid)
rm(list = ls(all = TRUE)) 
set.seed(10)

trucks_data = read.csv("truck_data2.csv")



#############################################################################
## Section 3.3
#############################################################################


## 3.3 "six, or 12%, of the 50 loads are below the 12.5 ton spec." 
## 3.3 count of # truck runs ("loads") below 12.5 tons

# Count the number of loads out of the 50 that are below the 12.5 spec
length(which(trucks_data$tons < 12.5)) == 6


# Fraction of loads below the 12.5 spec
length(which(trucks_data$tons < 12.5))/50 == 0.12

## "If we look at the six truck loads out of the 50 that are below the spec, 
## two are on Thursday and four on Friday. These two days have the lowest 
## average per-day tons."
below.spec <- trucks_data$tons < 12.5
day.below.spec <- trucks_data$day[trucks_data$tons < 12.5]
day.below.spec
unique(day.below.spec)
aggregate(trucks_data$tons ~ trucks_data$day, trucks_data, mean)




###################################
## Table 3. The data: loads in tons
###################################

##  The odd rows of the data frame, correspond to the "one" trucks.
one_trucks <- trucks_data[seq(from=1, to=49, by=2),]
two_trucks <- trucks_data[seq(from=2, to=50, by=2),]

table3.1 <- xtabs(tons ~ route + day, one_trucks)
table3.1

table3.2 <- xtabs(tons ~ route + day, two_trucks)
table3.2

cbind(table3.1, table3.2)



###################################
# Figure 1 - Frequency Distribution of Loads in Tons (Histogram)
###################################

# Figure 1: Frequency distribution of measured loads.
plot_frequency.distribution.of.measured.loads <- (
  ggplot(trucks_data,aes(x=tons))
  + geom_histogram(binwidth=.5, colour="black", fill="white")  
  +scale_y_continuous(name="Frequency", breaks=c(2,4,6,8,10,12,14))
  +coord_cartesian(xlim = c(11,16.5), ylim = c(0,13))
  +scale_x_discrete(name="Load in tons", breaks=c(11,12,13,14,15,16))
  +theme(axis.text=element_text(size=13,color="black"),
         axis.text.x=element_text(angle=0, hjust=1),
         axis.title=element_text(size=20,color=("black")),
         panel.background=element_blank(),
         panel.grid.major.y=element_line(color=rgb(.7,.7,.7),size=.01),
         panel.grid.minor.y=element_line(color=rgb(.9,.9,.9),size=.001),
         panel.grid.major.x=element_blank(),
         axis.line=element_line(size=.5),
         title=element_text(size=20,family="serif")
  )
)
plot_frequency.distribution.of.measured.loads



# Four missing values were imputed by least squares.These are observations
## at factor levels: 1/Tu/A1; 1/Tu/A2; 4/W/A1; and 4/M/D1. 
length(which(trucks_data$imputed > 0))

# Change route to a factor.
trucks_data$route = as.factor(trucks_data$route)



#############################################################################
# Section 4
#############################################################################


###################################
# Figure 2 - Loads by route, day, and truck
###################################

# Boxplots by Truck
trucks_data$truck <- factor(trucks_data$truck, 
    levels=c("A1", "B1", "C1", "D1", "E1", "A2", "B2", "C2", "D2", "E2"))
grand.mean <- mean(trucks_data$tons)

plot_boxplots.by.truck <- (
  ggplot(trucks_data,aes(factor(truck), tons)) + geom_boxplot()
  +scale_y_continuous(name="Load in tons")
  +scale_x_discrete(name="Truck")
  +theme(axis.text=element_text(size=13,color="black"),
         axis.text.x=element_text(angle=0, hjust=1),
         axis.title=element_text(size=20,color=("black")),
         panel.background=element_blank(),
         panel.grid.major.y=element_line(color=rgb(.7,.7,.7),size=.01),
         panel.grid.minor.y=element_line(color=rgb(.9,.9,.9),size=.001),
         panel.grid.major.x=element_blank(),
         axis.line=element_line(size=.5),
         legend.position="none",
         title=element_text(size=20,family="serif")
  )
  +geom_hline(yintercept=grand.mean, color="red")
)
plot_boxplots.by.truck

# Boxplots by  (Route)
plot_boxplots.by.route <- (
  ggplot(trucks_data,aes(factor(route), tons)) + geom_boxplot()
    +scale_y_continuous(name="Load in tons")
  +scale_x_discrete(name="Route")
  +theme(axis.text=element_text(size=13,color="black"),
         axis.text.x=element_text(angle=0, hjust=1),
         axis.title=element_text(size=20,color=("black")),
         panel.background=element_blank(),
         panel.grid.major.y=element_line(color=rgb(.7,.7,.7),size=.01),
         panel.grid.minor.y=element_line(color=rgb(.9,.9,.9),size=.001),
         panel.grid.major.x=element_blank(),
         axis.line=element_line(size=.5),
         legend.position="none",
         title=element_text(size=20,family="serif")
  )
  +geom_hline(yintercept=grand.mean, color="red")
)

plot_boxplots.by.route

# Boxplots by Day
trucks_data$day <- factor(trucks_data$day, levels=c("M", "T", "W", "TH", "F"))

plot_boxplots.by.day <- (
  ggplot(trucks_data,aes(factor(day), tons)) + geom_boxplot()
  #  +ggtitle("Figure 4: Plot of loads by day")
  +scale_y_continuous(name="Load in tons")
  +scale_x_discrete(name="Day")
  +theme(axis.text=element_text(size=13,color="black"),
         axis.text.x=element_text(angle=0, hjust=1),
         axis.title=element_text(size=20,color=("black")),
         panel.background=element_blank(),
         panel.grid.major.y=element_line(color=rgb(.7,.7,.7),size=.01),
         panel.grid.minor.y=element_line(color=rgb(.9,.9,.9),size=.001),
         panel.grid.major.x=element_blank(),
         axis.line=element_line(size=.5),
         legend.position="none",
         title=element_text(size=20,family="serif")
  )
  +geom_hline(yintercept=grand.mean, color="red")
)

plot_boxplots.by.day

#Put all 3 box plots in one figure
library(grid)
library(gridExtra)

final.boxplots <- grid.arrange(plot_boxplots.by.truck, 
  arrangeGrob(plot_boxplots.by.day, plot_boxplots.by.route, 
    heights=c(3/4, 1/4,1/4), ncol=2, nrow=1, ncol=2))


## Caption of Figure 2
## Grand.mean of loads is 13.71
round(mean(trucks_data$tons), 2) == 13.71




###################################
# Figure 3 -Residuals from fitted classical ANOVA model by truck. 
###################################
options(contrasts=c("contr.sum", "contr.poly"))
aov.three.way <- aov(tons ~ route + day + truck, data=trucks_data)
aov.three.way
summary(aov.three.way)
fitted(aov.three.way)

trucks_data$residuals <- trucks_data$tons - fitted(aov.three.way)
sum(trucks_data$residuals) == 0 ## check

plot_dot.plot.by.truck <- (  
  ggplot(trucks_data, aes(x = truck, y = residuals, 
      ymax=max(residuals))) + geom_point(binaxis = "y", stackdir = "center", 
          position = "dodge", cex=5, shape=43)
  +scale_y_continuous(name="Residuals")
  +scale_x_discrete(name="Truck")
  +coord_cartesian(ylim = c(-2,2))
  +theme(axis.text=element_text(size=13,color="black"),
         axis.text.x=element_text(size=20, angle=0, hjust=1),
         axis.text.y=element_text(size=20),
         axis.title=element_text(size=30,color=("black"), vjust=.35),
         panel.background=element_blank(),
         panel.grid.major.y=element_line(color=rgb(.7,.7,.7),size=.01),
         panel.grid.minor.y=element_line(color=rgb(.9,.9,.9),size=.001),
         panel.grid.major.x=element_blank(),
         axis.line=element_line(size=.5),
         title=element_text(size=20,family="serif")
  )
  + geom_hline(yintercept=0, color="red")  ## added to this code 6/11/2015
)
plot_dot.plot.by.truck


#################################################
##  Section 4.1
#################################################

## Grand mean
round(mean(trucks_data$tons), 2) == 13.71

## Standard deviation of the loads
round(sd(trucks_data$tons), 4) == 0.9788

## Standard error of the mean load 
round(sd(trucks_data$tons)/sqrt(50), 4) == 0.1384





## Means of the trucks
truck.means <- aggregate(trucks_data$tons ~ trucks_data$truck, trucks_data, mean)
names(truck.means) <- c("truck", "mean")
truck.means

round(range(truck.means$mean)[2] - range(truck.means$mean)[1], 2) == 0.94




## caption to Figure 4, last line

## Standard error of each truck mean, y..t

sigma.sq.hat <- 0.377  ## E(MSE) from ANOVA table of fixed effects model
s.e.truck.mean <- sqrt(sigma.sq.hat/5)
s.e.truck.mean  ## 0.2745906  Last line: sqrt(0.377/5) = 0.2745906





###################################
# Figure 4: Observed truck means with 50% and 90% confidence intervals. 
###################################

t.90.4 <- qt(0.95, 4) 
t.50.4 <- qt(0.75, 4)
ntruckruns <- 5
plot_truck.means.raw.data <- (
  ggplot(truck.means,aes(truck.means$truck, truck.means$mean))
  +geom_point(size=4)
  +geom_line(size=.71)
  +scale_y_continuous(name="Tons")
  +scale_x_discrete(name="Truck")
  +coord_cartesian(ylim=c(12.25,14.75))
  +theme(axis.text=element_text(size=17,color="black"),
         axis.text.x=element_text(size=20, angle=0,hjust=1),
         axis.text.y=element_text(size=20),
         axis.title=element_text(size=30,color=("black"), vjust=.35),
         panel.background=element_blank(),
         panel.grid.major.y=element_line(color=rgb(.7,.7,.7),size=.01),
         panel.grid.minor.y=element_line(color=rgb(.9,.9,.9),size=.001),
         panel.grid.major.x=element_blank(),
         axis.line=element_line(size=.5),
         title=element_text(size=20,family="serif"),
         legend.text=element_text(size=20),
         legend.title=element_text(size=20,face="bold"),
         legend.position=c(.05,.88))
  + geom_hline(yintercept=12.5, color="red")
  + geom_hline(yintercept=13.73, color="red")
  + annotate("text", x = 8.25, y = 14.45, label = "Mean load", size=6)
  + geom_segment(aes(x = 8.25, y = 14.35, xend = 7.25, yend = 13.73), arrow = arrow(length = unit(0.5, "cm")))
  + annotate("text", x = 5, y = 12.85, label = "Contract spec", size=6)
  + geom_segment(aes(x = 5, y = 12.75, xend = 4, yend = 12.52), arrow = arrow(length = unit(0.5, "cm")))
  +annotate("segment", x = 1, xend = 1, y = truck.means$mean[1]-(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[1]+(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)))
  +annotate("segment", x = 2, xend = 2, y = truck.means$mean[2]-(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[2]+(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)))
  +annotate("segment", x = 3, xend = 3, y = truck.means$mean[3]-(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[3]+(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)))
  +annotate("segment", x = 4, xend = 4, y = truck.means$mean[4]-(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[4]+(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)))
  +annotate("segment", x = 5, xend = 5, y = truck.means$mean[5]-(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[5]+(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)))
  +annotate("segment", x = 6, xend = 6, y = truck.means$mean[6]-(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[6]+(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)))
  +annotate("segment", x = 7, xend = 7, y = truck.means$mean[7]-(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[7]+(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)))
  +annotate("segment", x = 8, xend = 8, y = truck.means$mean[8]-(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[8]+(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)))
  +annotate("segment", x = 9, xend = 9, y = truck.means$mean[9]-(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[9]+(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)))
  +annotate("segment", x = 10, xend = 10, y = truck.means$mean[10]-(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[10]+(t.90.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)))

  +annotate("segment", x = 1, xend = 1, y = truck.means$mean[1]-(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[1]+(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), size = 1.75)
  +annotate("segment", x = 2, xend = 2, y = truck.means$mean[2]-(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[2]+(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), size = 1.75)
  +annotate("segment", x = 3, xend = 3, y = truck.means$mean[3]-(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[3]+(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), size = 1.75)
  +annotate("segment", x = 4, xend = 4, y = truck.means$mean[4]-(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[4]+(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), size = 1.75)
  +annotate("segment", x = 5, xend = 5, y = truck.means$mean[5]-(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[5]+(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), size = 1.75)
  +annotate("segment", x = 6, xend = 6, y = truck.means$mean[6]-(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[6]+(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), size = 1.75)
  +annotate("segment", x = 7, xend = 7, y = truck.means$mean[7]-(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[7]+(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), size = 1.75)
  +annotate("segment", x = 8, xend = 8, y = truck.means$mean[8]-(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[8]+(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), size = 1.75)
  +annotate("segment", x = 9, xend = 9, y = truck.means$mean[9]-(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[9]+(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), size = 1.75)
  +annotate("segment", x = 10, xend = 10, y = truck.means$mean[10]-(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), yend = truck.means$mean[10]+(t.50.4*sqrt(sigma.sq.hat)/sqrt(ntruckruns)), size = 1.75)
)
plot_truck.means.raw.data





# Q-Q plot indicates normality of residuals (noted, but not displayed in the document)
qqpnorm(residuals(aov.three.way))

## Table 4 ANOVA tabel of results
anova(aov.three.way)

## Fixed effects estimates
model.tables(aov.three.way)


## Table 5. Estimated fixed effects for the 3 factors

## str(model.tables(aov.three.way))



## Calculate sample variances and standard errors of each of 
## the 3 sets of fixed effects.
## Last paragraph of section 4.1
## and Table 6.
var((model.tables(aov.three.way))[[1]][[1]])  ## route effects
var((model.tables(aov.three.way))[[1]][[2]])  ## day effects
var((model.tables(aov.three.way))[[1]][[3]])  ## truck effects;  value = 0.08331111
round(sqrt(var((model.tables(aov.three.way))[[1]][[1]])), 2) == 0.43
round(sqrt(var((model.tables(aov.three.way))[[1]][[2]])), 2) == 0.77
round(sqrt(var((model.tables(aov.three.way))[[1]][[3]])), 2) == 0.29




#############################################################################
## Section 4.2
#############################################################################


## Mixed effects model estimates
library(nlme)
require(nlme)

trucks_data$Block <- rep(1,nrow(trucks_data))

truckGrouped <- groupedData(tons ~ 1 | Block, dat=trucks_data)
my.fit <- lme(tons ~ day, random = pdBlocked(list(pdIdent(~ route-1),pdIdent(~ truck-1))), 
              dat=truckGrouped)
summary(my.fit)




###################################
## Table 7 Variance components of factors in the random effects model estimated 
## by (restricted) maximum likelihood using nmle
###################################

## Use nmle to find 3 variance components and error and square roots of them

library(nlme)
require(nlme)

trucks_data$Block <- rep(1,nrow(trucks_data))

truckGrouped <- groupedData(tons ~ 1 | Block, dat=trucks_data)
me1 <- lme(tons ~ 1, random = pdBlocked(list(pdIdent(~ route-1),
                  pdIdent(~ day-1), pdIdent(~ truck-1))), dat=truckGrouped)

sigma.4 <- unname(exp(attr(me1$apVar,"Pars"))) ## roots of variance components and error
sigma.4  ## Result: [1] 0.38129713 0.74658390 0.08880375 0.61410504

##  Compare to results when days are a fixed effect
unname(exp(attr(my.fit$apVar,"Pars")))   ##  Identical


sigma.4^2  ## variance components and error




#############################################################################
# Section 4.3
#############################################################################

## Figure 5.
###################################
## Simulation to find confidence intervals for roots of 3 variance components and error
###################################

nsim <- 20000  ## N.B. The simulation takes several hours to run.
trucks_data.sim.mat <- matrix(NA, 4*nsim)
dim(trucks_data.sim.mat) <- c(nsim, 4)

for (i in 1:nsim) {
  ## Draw 4 random vectors for effects and error
  gamma.r <- rnorm(5, 0, sigma.4[1])
  tau.d <- rnorm(5, 0, sigma.4[2])
  rho.t <- rnorm(10, 0, sigma.4[3])
  error <- rnorm(50, 0, sigma.4[4])

  ## Draw vector y of tons loaded of length 50 according to model (1) in text.
  y <- numeric(50)
  for(j in 1:50)
     y[j] <- grand.mean + gamma.r[trucks_data$route[j]] + tau.d[trucks_data$day[j]] + rho.t[trucks_data$truck[j]] + error[j]

  ## Define the simulated data set analogous to original trucks_data
  trucks_data.sim <- cbind(y, trucks_data[2:4])

  ## Use nmle to find 3 variance components and error variance.
  trucks_data.sim$Block <- rep(1,nrow(trucks_data.sim))
  truckGrouped <- groupedData(y ~ 1 | Block, dat=trucks_data.sim)
  me1 <- try( lme(y ~ 1, random = pdBlocked(list(pdIdent(~ route-1),
                    pdIdent(~ day-1), pdIdent(~ truck-1))), dat=truckGrouped, 
                    control=list()) , silent=TRUE)
  if ('try-error' %in% class(me1)) next

  if(is.numeric(me1$apVar))
    trucks_data.sim.mat[i, ] <- unname(exp(attr(me1$apVar,"Pars")))

  if(i %% 100 == 0)  print(i)
}

## Get c(.05, .95) percentiles of the nsim simulated roots--for each of the 3 roots.
## These are endpoints of the 90% confidence intervals for the roots of
## the true variance components--as shown in Figure 5 of paper.

quantile(trucks_data.sim.mat[,1], probs = c(0.05, 0.95), na.rm = TRUE)
quantile(trucks_data.sim.mat[,2], probs = c(0.05, 0.95), na.rm = TRUE)
quantile(trucks_data.sim.mat[,3], probs = c(0.05, 0.95), na.rm = TRUE)



###################################
# Figure 5  Standard deviation of effects--the square root of a variance component--
# and 50% and 90% confidence intervals for them. 
###################################


st.dev.components.w.confidence <- data.frame("count" = c("Route", "Day", "Truck"), 
    "components" = c(sigma.4[1], sigma.4[2], sigma.4[3]), 
    "lower90" = c(quantile(trucks_data.sim.mat[,1], probs = 0.05, na.rm = TRUE),
                quantile(trucks_data.sim.mat[,2], probs = 0.05, na.rm = TRUE),
                quantile(trucks_data.sim.mat[,3], probs = 0.05, na.rm = TRUE)), 
    "upper90" = c(quantile(trucks_data.sim.mat[,1], probs = 0.95, na.rm = TRUE),
                quantile(trucks_data.sim.mat[,2], probs = 0.95, na.rm = TRUE),
                quantile(trucks_data.sim.mat[,3], probs = 0.95, na.rm = TRUE)),
    "lower50" = c(quantile(trucks_data.sim.mat[,1], probs = 0.25, na.rm = TRUE),
                quantile(trucks_data.sim.mat[,2], probs = 0.25, na.rm = TRUE),
                quantile(trucks_data.sim.mat[,3], probs = 0.25, na.rm = TRUE)), 
    "upper50" = c(quantile(trucks_data.sim.mat[,1], probs = 0.75, na.rm = TRUE),
                quantile(trucks_data.sim.mat[,2], probs = 0.75, na.rm = TRUE),
                quantile(trucks_data.sim.mat[,3], probs = 0.75, na.rm = TRUE))
)

st.dev.components.w.confidence$count <- factor(st.dev.components.w.confidence$count, 
                                              levels = c("Route", "Day", "Truck"))

plot_vertical.st.dev.confidence <- (
  ggplot(st.dev.components.w.confidence, aes(x = count, y = components)) +
    geom_point(size = 4)
  +theme(axis.ticks.x=element_blank())
  +scale_x_discrete(name="Factor")
  +scale_y_continuous(name="Standard deviation of effects")
  +coord_cartesian(ylim=c(0,1.075*max(st.dev.components.w.confidence$upper90)), xlim=c(0,4))
  +theme(axis.text=element_text(size=17,color="black"),
         axis.text.x=element_text(size=24, angle=0),
         axis.text.y=element_text(size=24),
         axis.title=element_text(size=30,color=("black"), vjust=.75),
         panel.background=element_blank(),
         panel.grid.major.y=element_line(color=rgb(.7,.7,.7),size=.01),
         panel.grid.minor.y=element_line(color=rgb(.9,.9,.9),size=.001),
         panel.grid.major.x=element_blank(),
         axis.line=element_line(size=.5),
         title=element_text(size=20,family="serif"),
         legend.text=element_text(size=20),
         legend.title=element_text(size=20,face="bold"),
         legend.position=c(.05,.88),
         axis.title.y=element_text(vjust=1.5),
         axis.title.x=element_text(vjust=-0.5))
  +annotate("segment", x = 1, xend = 1, y = st.dev.components.w.confidence$lower90[1], 
            yend = st.dev.components.w.confidence$upper90[1])
  +annotate("segment", x = 2, xend = 2, y = st.dev.components.w.confidence$lower90[2], 
            yend = st.dev.components.w.confidence$upper90[2])
  +annotate("segment", x = 3, xend = 3, y = st.dev.components.w.confidence$lower90[3], 
            yend = st.dev.components.w.confidence$upper90[3])
  +annotate("segment", x = 1, xend = 1, y = st.dev.components.w.confidence$lower50[1], 
            yend = st.dev.components.w.confidence$upper50[1], size = 1.75)
  +annotate("segment", x = 2, xend = 2, y = st.dev.components.w.confidence$lower50[2], 
            yend = st.dev.components.w.confidence$upper50[2], size = 1.75)
  +annotate("segment", x = 3, xend = 3, y = st.dev.components.w.confidence$lower50[3], 
            yend = st.dev.components.w.confidence$upper50[3], size = 1.75)
)

plot_vertical.st.dev.confidence


###################################
# Note. Calculation of the posterior densities and statistics for them are given
# in file: "R Code for Section 5 - A Bayesian Perspective.r"
###################################


